;Purpose
; 	To read and write to the internal eeprom
;Entry Functions
;	EEWriteByteZ	e2(Z)=al  Writes 1 byte to the internal e2  	uses dh
;	EEReadByteZ	al=e2(Z)  Reads 1 byte from the internal e2 	uses dh
;	EEWriteStrYZ	e2(Z)=ram(y)*ah Writes a string of bytes    	uses ah,dh
;	EEReadStrYZ	ram(Y)=e2(Z)*ah Reads a string of bytes		uses ah,dh
;
;uses 
;	rom = 38 words
;may-10-98 created by bill Arden from app note
;Sep-02-02 changed to YZ since X is needed for main
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
EEWriteStrYZ:
;	Purpose
;		e2(Z)=ram(y)*ah Writes a string of bytes
;	in
;		Y = Ram location to read from
;		Z = e2 location to write to
;		ah = number of words to write
;	out
;		ah = 0
;		Y = Past end of buffer
;		Z = Past end of buffer
;	uses
;		ax,dh
;	stack = 4 (call to this subroutine) and call to EEWriteByte

EEWriteStrLoop:

	rcall	EEReadByteZ	;al=e2(Z) uses dh	;read 
	ld	dh,Y+		;Get byte and point to next
	cp	al,dh
	breq	EEWriteStrNotNeeded			;A write is not needed

	mov	al,dh		;
	rcall	EEWriteByteZ	;e2(Z)=al uses dh
EEWriteStrNotNeeded:
	adiw	ZL,1		;next e2 byte

	dec	ah		;loop until done
	brne	EEWriteStrLoop	

	ret
;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

;	Purpose
;		To Write 1 byte to the internal e2
;	in
;		al = byte to write
;		Z = e2 location
;	out
;		al = unchanged
;		Z = unchanged
;	uses
;		dh
;	stack = 2 (call to this subroutine)
;
EEWriteByteRetry:
;	out	sreg,dh

EEWriteByteZ:

;This subroutine waits until the EEPROM is ready to be programmed, then			
EEWriteByteLoop:
	sbic	EECR,EEWE		;if EEWE not clear (bit 1)
	rjmp	EEWriteByteLoop		;wait more

;	in	dh,sreg
;	cli				;no interupts
;	sbic	EECR,EEWE		;if EEWE not clear (bit 1)
;	rjmp	EEWriteByteRetry	;wait more but pop the stack

;@ programs the EEPROM with register variable "AL" at address "ZH, ZL"
	out	EEARH,ZH		;output address high first
	out	EEARL,ZL		;output address low
	out	EEDR,AL			;output data

;	in	dh,sreg
	cli				;1,1  no interupts

;@ Interrupt between setting EEMWE and setting EEWE will prevent a write,
;@ because EEMWE is automatically cleared after 4 clock cycles, and must
;@ be set to enable a write.
	sbi	EECR,EEMWE		;1,2 set Master Write Enable
	sbi	EECR,EEWE		;1,4 set EEPROM Write strobe
					;    This instruction takes 4 clock
					;    cycles since it halts the CPU for
					;    two clock cycles
	sei				;1,1 Enable interrupts
;	out	sreg,dh			;1,1 reenable interupts if they were enabled

;EEWriteAbort:
	ret

;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@
EEReadStrYZ:	
;	Purpose
;		ram(Y)=e2(Z)*ah Reads a string of bytes
;	in
;		Z = e2 location to read from
;		Y = Ram location to write to
;		ah = number of words to read
;	out
;		ah = 0
;		Y = Past end of buffer
;		Z = Past end of buffer
;	uses
;		ah,dh
;	stack = 4 (call to this subroutine) and call to EEReadByte
;
;rcall	EEReadStr ;ram(x)=e2(y)*ah  uses ah,dh
	
EEReadStrLoop:
	
	rcall	EEReadByteZ	;al=e2(Z) uses dh
	adiw	ZL,1		;next e2 byte
	st	Y+,al		;store byte and next

	dec	ah		;loop until done
	brne	EEReadStrLoop	


;@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@@

 
;	purpose
;		to Read 1 byte from the internal e2
;	in
;		Z = e2 address
;	out
;		al = byte
;		Z = unchanged
;	uses
;		dh
;	stack = 2 bytes (call to this subroutine)
EEReadByteRetry:
;	out	sreg,dh

EEReadByteZ:

;@ This subroutine makes sure that the EEPROM isn't busy writing, then
EEReadByteLoop:
	sbic	EECR,EEWE		;if EEWE not clear
	rjmp	EEReadByteLoop		;wait more

;	in	dh,sreg
;	cli				;no interupts
;	sbic	EECR,EEWE		;if EEWE not clear (bit 1)
;	rjmp	EEReadByteRetry		;wait more but pop the stack

;@ reads the register variable "AL" from address "YH, YL"
	out	EEARH,ZH		;output address high first
	out	EEARL,ZL		;output address low

;	in	dh,sreg
;	cli				;1,1 no interupts

	sbi	EECR,EERE		;1,4 set EEPROM Read strobe
					;    This instruction takes 4 clock cycles since
					;    it halts the CPU for two clock cycles
	in	AL,EEDR			;    get data

;	out	sreg,dh			;    reenable interupts if they were enabled

	ret
