;***************************************************************************;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y;* ;* Number		:AVR020
;* File Name		:"PushA.mac"
;* Title		:Push and Pop  Macro Definitions
;* Date	 		:april-18-98
;* Version 		:1.00
;* Programmer		: Bill Arden


;***************************************************************************
.macro PUSH1
	in	SREG_s,sreg
	push 	al	;same as temp
	push	ah
.endmacro	
.macro POP1 			;POP The Ax registers
	pop	ah
	pop 	al
	out	sreg,SREG_s
.endmacro

;Example ->	Push1		;Push Ax and sreg
;Example ->	POP1 		;Pop  Ax and SREG	

;****************************************************************************
.macro PUSH2
	in	SREG_s,sreg
	push 	al	;same as temp
	push	ah
	push	bl
	push	bh

.endmacro	
.macro POP2 			;POP The Ax and BX registers
	pop	bh
	pop	bl
	pop	ah
	pop 	al
	out	sreg,SREG_s
.endmacro
;***************************************************************************
.macro PUSH4
	in	SREG_s,sreg
	push 	al
	push	ah
	push	bl
	push	bh
	push	cl
	push	ch
	push	dl
	push	dh
.endmacro	
.macro POP4 	
	pop	dh
	pop	dl
	pop	ch
	pop	cl
	pop	bh
	pop	bl
	pop	ah
	pop 	al
	out	sreg,SREG_s
.endmacro

;reentrant Versions
.macro PUSH4R
	push 	al
	in	al,sreg
	push 	al
	push	ah
	push	bl
	push	bh
	push	cl
	push	ch
	push	dl
	push	dh
.endmacro	
.macro POP4R
	pop	dh
	pop	dl
	pop	ch
	pop	cl
	pop	bh
	pop	bl
	pop	ah
	pop 	al
	out	sreg,al
	pop 	al
.endmacro

;Example ->	Push4		;Push Ax,bx,cx,dx and sreg
;Example ->	POP4 		;Pop  Ax,bx,cx,dx and SREG	


;***************************************************************************
.macro PUSHX
	push 	xl
	push	xh
.endmacro	
.macro POPX 	
	pop	xh
	pop	xl
.endmacro

.macro PUSHY
	push 	yl
	push	yh
.endmacro	
.macro POPY 			
	pop	yh
	pop	yl
.endmacro
.macro PUSHz
	push 	zl
	push	zh
.endmacro	
.macro POPz		
	pop	zh
	pop	zl
.endmacro


;*************************************************************************** 

;* A P P L I C A T I O N   N O T E   F O R   T H E   A V R   F A M I L Y
;* Number		:AVR020
;* File Name		:"imm16.mac"
;* Title		:16-Bit Immediate Operations Macro Definitions
;* Date	 		:96.07.08
;* Version 		:1.01
;* Support telephone	:+47 72 88 87 20 (ATMEL Norway)
;* Support fax		:+47 72 88 87 18 (ATMEL Norway)
;* Support E-mail	:support@atmel.no
;* Target MCU		:AT90S2312,AT90S8414
;* DESCRIPTION
;* This is a macro definition file containing macros for 16-bit immediate
;* operations. 
;* In addition, the macro ADDI is defined. This macro enables addition of an 
;* 8-bit immediate value to a register.
;***************************************************************************
.macro	ADDI			;Adds an 8 bit immediate to a register
	subi	@0,-@1		;subtract negative value
.endmacro
;Example ->	ADDI	r20,$55		;add the value $55 to r20
;***************************************************************************
.macro	ADDI16			;Adds a 16 bit immediate to a 
				;16 bit register variable
	subi	@1,low(-@2)	;subtract negative low byte
	sbci	@0,high(-@2)	;subtrac negative high byte with carry
.endmacro
;Example ->	ADDI16	r21,r20,$4b32	;add the value $4b32 to r21:r20
;***************************************************************************
.macro	SUBI16			;Subtracts a 16 bit immediate from
				;a 16 bit register variable
	subi	@1,low(@2)	;Subtract low byte
	sbci	@0,high(@2)	;Subtract high byte with carry
.endmacro
;Example ->	SUBI16	r21,r20,10000	;subtracts 10000 from r21:r20

;***************************************************************************

.macro	LDI16			;Loads a 16 bit immediate to a 16 bit 
				;register variable
	ldi	@1,low(@2)	;Subtract low byte
	ldi	@0,high(@2)	;Subtract high byte with carry
.endmacro

;Example ->	LDI16	r17,r16,-4555	;loads r17:r16 with -4555

;***************************************************************************


.macro LDX 			;Load the X-pointer with a 16 bit value
	ldi 	XL,low(@0)	;Load low byte
	ldi 	XH,high(@0)	;Load high byte
.endmacro
.macro LDY 			;Load the Y-pointer with a 16 bit value
	ldi 	YL,low(@0)	;Load low byte
	ldi 	YH,high(@0)	;Load high byte
.endmacro
.macro LDZ 			;Load the Z-pointer with a 16 bit value
	ldi 	ZL,low(@0)	;Load low byte
	ldi 	ZH,high(@0)	;Load high byte
.endmacro

;****************************************************************************
.macro Timer8x
;
;	Purpose
;		to create a 8 bit varible timer
;	in
;		X
;		@0 = timer count. t= @1 to @2-1
;	out
;		X = +1
;		@0 = ram(X)
;	usage
;		Timer8 al,0,7	;al = 0 to 6 ; timer al=ram(X+)=ram(X)+1 if >= ah then=0
;	uses
;		6 words

	ld	@0,X			;PLCR_MEM_Freq timer  120hz/12hz = 10
	inc	@0
	cpi	@0,@2
	brlo	Timer8not		; if al(u) >= ah(u) then 
	ldi	@0,@1			;al=0
Timer8not:
	st	X+,@0

.endmacro

;---------- math ------------------------

;--------------------- neg = twos complement 
;  neg16 H,L
;  neg24 H,M,L

.macro neg16
	com	@0
	com	@1
	subi	@0,1
	sbci	@1,0
.endmacro

.macro neg24
; twos complement of a 24 bit register

	com	@2	;Low
	com	@1	;M
	com	@0	;H

	subi	@2,1	;L
	sbci	@1,0	;M
	sbci	@0,0	;H
.endmacro

;--------------------------- ABS absolute value
;abs16
;abs24

.macro ABS16
	cpi	@0,0		;Sgn of high
	brge	ABS16_notInvert	;if < 0 then
	com	@0		;invert
	com	@1
	subi	@1,1		;L
	sbci	@0,0		;H
ABS16_notInvert:
.endmacro
.macro ABS24
	cpi	@0,0		;Sgn of high
	brge	ABS24_notInvert	;if < 0 then

	com	@2	;Low
	com	@1	;M
	com	@0	;H

	subi	@2,1	;L
	sbci	@1,0	;M
	sbci	@0,0	;H
ABS24_notInvert:
.endmacro

;--------------------------- Saturate value into x bits
; Saturate24To16 H,M,L,temp   ; take 24 bit number and saturate it into 16 bits. 24 bit output
.macro Saturate24To16
	clr	@3		;temp
	cpi	@1,0x80		;M
	cpc	@0,@3		;H compare H,M to 0080
	brlt	Saturate24To161N
	clr	@0		;H
	ldi	@1,0x7F		;M
	ldi	@2,0xFF		;L
Saturate24To161N:
	ldi	@3,0xff		;temp
	cpi	@1,0x81		;M
	cpc	@0,@3		;H compare H,M to FF81
	brge	Saturate24To162n
	ldi	@0,0xff		;H
	ldi	@1,0x81		;M
	ldi	@2,0x00		;L
Saturate24To162N:


.endmacro
;--------------------------------------- SignExtend8 sign extend number by 8 bits
; SignExtend8 H,L

.macro SignExtend8
	clr	@0		;assume positive
	cpi	@1,0
	brge	signextend8notneg
	dec	@0		;if negitive then change H to 0xff
signextend8notneg:

.endmacro

;-------------------------------------- Send a out using al
.macro OUTI
	ldi	al,@1
	out	@0,al
.endmacro


;-------------------------------------- Bit test and jump
;Jump if bit is clear al,0,lable
.macro	JumpBitClear
	sbrs	@0,@1
	rjmp	@2
.endmacro

;Jump if bit is set al,0,lable
.macro 	JumpBitSet
	sbrc	@0,@1
	rjmp	@2	
.endmacro

;Move a bit TO,b ,from,b
.macro MoveBit
	BST	@2,@3	;T = from(x)
	BLD	@0,@1	;To(x) = T
.endmacro
