; Atmel Servo R/C controll
; for the AT90S2313 processor.
; By Bill arden www.longrange.net
;

.equ ProgramVersion = 0x00
.equ ProgramType = 0x00

;V0,0 Dec 7 2002 started

;Main.asm 	coordinates system
; +- QuadIn.asm	Quadrature Decoder (EXT_INT0:,EXT_INT1:)
; +- SerPkt.asm	Rs232 Serial Packet handler (UART_RXC:,UART_DRE:,UART_TXC:)

;Port B
; PB0 = Ain0	= R/C Input 
; PB1 = Ain1 	= Threshhold for R/C input
; PB2 =	 	= Home switch
; PB3 = oc1 	= R/C output or PWM output
; PB4 = 	= PWM direction 
; PB5 = MoSi	= Tx_en
; PB6 = MiSo	= Limit1
; PB7 = SCK	= Limit2
;Port D
; PD0 = RXD 	= Serial Rx (Active low) 
; PD1 = TXD 	= Serial Tx (Active low)
; PD2 = Int0 	= Quadrature B input
; PD3 = Int1 	= Quadrature B input again
; PD4 =	T0 	= Quadrature A input
; PD5 = T1 	= Select In
; PD6 = ICP 	= Select Out

.include "2313def.inc"	;define ports and XH,XL,YH,YL,ZH,ZL
.include "Xregs.inc"	; A,B,C,D and temp=AL tempory registors r16-r31(hex)
.include "Macro.inc"	;push and pop macros

;Define our Hardware
.include "Hardware.inc"
.equ	Processor_Clock_Freq = 9216000 		;clock frequency

.equ	Timer0_Freq_Freq = 1953   ;Hz xtal/1953=4718.89 /64 = 73.7; xtal/ (73 * 64) = 1972.6
.equ	Timer0_Freq_Ctr = 73   ;Hz xtal/1953=4718.89 /64 = 73.7; xtal/ (73 * 64) = 1972.6


;registers 2 to 15 
.def	UFlags = R2		;flags
.equ 	UFlag_MainFlag 	= 0	; .0 = 120 hz
.equ	UFlag_Power 	= 4	; .4 = 1 if power is on
.equ	UFlag_PowerLost = 5	; .5 = 1 when power is lost
.equ	UFlag_CarrierData =6 	; .6 = 1 for higher frequency
				; .7 = Carrier state
.def REG_Int0sreg = r3

.def REG_SerRXCsreg = r4
.def REG_SerDREsreg = r5
.def REG_SerTXCsreg = r6

;registers 16 to 31
;Ax,BX,CX,DX R16-R23
.def REG_EncoderL = r24		;also known as w or Ex
.def REG_EncoderH = r25
.def REG_EncodeAL = r26		;also known as X
.def REG_EncoderState = r27	

;---------------- Include ------------
	.include "e2_Def.inc"		;e2 flash definitions

.DSEG			
;NOTE: words are MSO, however they are sent LSO over the serial port!

	Ram_Status:		.byte 1		;[1]
		;Bit0 0x01 move_done	Clear if a trapezoidal move is in progress
					;Set if motor is not moving or if PID is disabled.
		;Bit1 0x02 cksum_error 	Set if there was a checksum error in the previous command
					;packet. (If the checksum is wrong there is no reply)
		;Bit2 0x04 overcurrent 	;(not used) =0
		;Bit3 0x08 power_on 	;(Not used) = 1
		;bit4 0x10 pos_error 	;Set if the position error exceeds the position error limit.
					;Set if PID is disabled.
					;Must be cleared using the clear sticky bits" command.
		;Bit5 0x20 Limit1 	;N/A =0
		;Bit6 0x40 Limit2 	;N/A =0
		;Bit7 0x80 HomeInProgress ;N/A =0
	Ram_StatusPos:		.byte 4 	;[4]  Pos 4 bytes Current encoder Position
	Ram_StatusAD:		.byte 1		;[1] A/D (R/C input) value
	Ram_StatusVel:		.byte 2		;[2]  Current velocity
	Ram_StatusAux:		.byte 1		;[1]  Auxiliary status byte 
		;Bit0 0x01 Index. Inverted value of the encoder index input.
		;Bit1 0x02 pos_wrap Set if the 32 bit position counter wraps or overflows.
		;		Must be cleared using the clear sticky bits" command.
		;Bit2 0x04 servo_on. 1 if the position servo is enabled.
		;Bit3 0x08 accel_done Set when the initial acceleration phase of a trapezoidal
		;		profile move is complete.
		;		Cleared when the next move is started.
		;Bit4 0x10 slew_done Set to 1 when the slew? portion of a trapezoidal profile
				;move is complete.
				;Cleared when the next move is started.
		;Bit5 0x20 servo_overrun Set to 1 when processor runs out of processing time.
				;Cleared when the next move is started.
		;Bit6 0x40 =0
		;Bit7 0x80 =0
	Ram_StatusHome:		.byte 4		;[4] Home position
	Ram_StatusDevType: 	.byte 2		;[2] Device type =0

;************************************************************
.CSEG

;-------------------------------------------------------------
	.org $0000
	rjmp	RESET		;Reset Handle
	rjmp	EXT_INT0
	rjmp	EXT_INT1
	rjmp	TIM_CAPT1
	rjmp	TIM_COMP1
	rjmp	TIM_OVF1
	rjmp	TIM_OVF0
	rjmp	UART_RXC
	rjmp	UART_DRE
	rjmp	UART_TXC
	rjmp	ANA_COMP

;***************************************************************
;EXT_INT0:,EXT_INT1: 			QuadIn.asm
TIM_OVF1:
TIM_OVF0:
;UART_RXC:,UART_DRE: ,UART_TXC:  	SerPkt.asm
ANA_COMP:
TIM_CAPT1:
TIM_COMP1:

	RETI;

;push	ah

;------------------ Timer 1----------------
;	PUSH2   ;push AX,BX and sreg 
;	PUSH4   ;push AX,BX,CX,DX and sreg 
;	POP2	;restore ax,bx and sreg
;	reti;
;------------------------------------------------------------------------------
;	popz
;	popx	;restore X
;	POP4	;restore ax,bx,cx,dx and sreg
;	reti	;
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;	Push2		;Push Ax, Bx and sreg
;	SetOutputTestHigh
;	POP2 		;Pop  Ax, Bx and SREG	
;	reti
;---------------------------------------------------------
RESET:
	clr	ah
	out	SREG,ah
;Stack
	ldi	al,RAMEND	;end of ram
	out	SPL,al		;init Stack Pointer


	clr	bh
	clr  	bl
	clr	ch
RamClear:
 	wdr
	clr	zh
	ldi	zl,0x60		;start of ram
	ldi	al,RAMEND-0x60	;number of bytes
	clr	ah
RamClearLoop:
	st	z+,ah		;clear ram 
	dec	al
	brne	RamClearLoop
	dec	bh
	brne	RamClear	;Startup delay
;	dec	bl
;	brne	RamClear
;	dec	ch
;	brne	RamClear

; reg clear
	clr	zh
	clr	zl
	clr	yh
	ldi	yl,18		;number of regs-4
RegClearLoop:
	st	z+,yh		;clear regs
	dec	yl
	brne	RegClearLoop
	
;-- memory test markers
;	ldi	al,ProgramType
;	sts	Ram_ProgramType,al		;program type
;	ldi	al,ProgramVersionConst
;	sts	Ram_ProgramVersion,al	;program version

;	ldi	al,0x55
;	sts	MyStack,al
;	ldi	al,0xba
;	sts	endofram,al

;--- analog 
;	ldi	al,0b00000100		;T1 latch on comparitor
;	out	ACSR,al
 
;Ports
	MacroSetupOutput		;hardware.h


;interupt mask registers. Init subs will enable any others that are needed
	clr	ah			;for latter

	ldi	al,0b00000000		;
	out	MCUCR,al		;MCU control

	;out	GIFR,ah			;IRQ flag External
	ldi	al,0b00000000	
	out	GIMSK,al		;IRQ mask External

	out	TIFR,ah			;IRQ flag timer
	
;	ldi	al,(1<<TICIE)		;Timer 1 input Latch IRQ enabled

	LDI	AL,0			;clear all timer IRQ's
	out	TIMSK,al

;	in	al,TIMSK
;	ori	al,(1<<TOIE1)		;Timer1 Overflow IRQ enabled
;	out	TIMSK,al

; Timer 0 1.953Khz source

	ldi	al,(1<<CS01)|(1<<CS00)	; = clk/64
	out	TCCR0,al		;Timer 0 control:  ;	in	al,TIMSK;	ori	al,(1<<TOIE0)		;Timer0 Overflow IRQ enabled;	out	TIMSK,al
	ldi	al,0			;clear timer (up counter)
	out	TCNT0,al

	
; Timer 1 ----- PWM or R/C in and R/C out ----

;	in	al,TIMSK
;	ori	al,(1<<OCIE1A)		;Timer1 output Compare Match A IRQ enabled
;	out	TIMSK,al

;	out	TCNT1H,ah		;Timer 1 Count High =0
;	out	TCNT1L,ah		;Timer 1 Count Low =0

;	ldi	ah,high(Processor_Clock_Freq/Timer_Freq_Freq)		; 10 bit 400h+11 for cap discharge
;	ldi	al,low(Processor_Clock_Freq/Timer_Freq_Freq) 		;Processor_Clock_Freq/PLCR_AD_RATE=0x411

;	out	OCR1AH,ah		;Timer 1 compare High
;	out	OCR1AL,al		;Timer 1 Compare Low

;	ldi	al,0b01000000		;Toggle oc1, no PWM
;	out	TCCR1A,al		;Timer 1 Control A
;	ldi	al,0b11001001		;deglitch,rising,,clear,clk Prototype
;	ldi	al,0b10001001		;deglitch,rising,,clear,clk Rev A
;	out	TCCR1B,al		;Timer 1 Control B

;registers

; IFR
	LDI	al,0b00001101
	out	WDTCR,al

;	LDX	Ram_ProgramVersion	;Load first 32 locations into ram
;	LDY	Boot_E2_Version
;	ldi	ah,32
;	rcall	EEReadStr ;ram(x)=e2(y)*ah  uses ah,dh

	rcall	QUADInInit	;configure encoder input

	rcall	SerPktInit	; configure serial port

	rcall	PIDInit		;PID.asm 
	rcall	MoveInit	;move.asm

;.equ	EEAR	
;.equ	EEDR	
;.equ	EECR	
;.equ	UBRR
;.equ	ACSR

;****************************************************************
	sei			; Enable interrupts
MAIN:
	
MainLoop1:
	rcall	QuadReadCounter ;get encoder value
	rcall 	SerPktmain	;Check for packet

;	sleep		;
 	wdr

	in	al,TCNT0		;get timer 0 (up counter)
	cpi	al,Timer0_Freq_Ctr	;=73
	brlo	MainLoop1
	ldi	al,0			;clear timer (up counter)
	out	TCNT0,al
;--- Get here at 1972.6 hz rate ----------------
	


;	sbrs	UFlags,0
;	rjmp MainLoop1
;	dec	Uflags

	rjmp	main
;****************************************************************

;.equ	AtBCD0	=13		;address of tBCD0
;.def	tBCD0	=r13		;BCD value digits 1 and 0
;.def	tmp16a	=r19		;temporary value
;-------------------------------------------------------
;	.include "Accm24.asm"	;16 bit Accumulator
;
;------------------------------------------------------

.include "QuadIn.asm"		;quadrature decoder
.include "SerPkt.asm"		;Serial Packet Decoder
.include "PID.asm"		;
.include "Move.asm"		;

.include "E2_int.asm"		; Internal E2

.DSEG				
endofram:	.byte 1
.org ramend-31
MyStack: 	.byte 32	;stack space
.cseg
