; Interrupt quadrature decoder
; By Bill arden www.longrange.net
; 
;
;Entry
;	QUADInInit:
;	QuadClearCounter:
;	QuadReadCounter:
;	EXT_INT0:
;	EXT_INT1:
;Pins
;	PD2,3 quadrature B input (Int0 and INT1)
;	PD4 quadrature A input 
;uses
;	REG_Int0sreg	;r00-r31; (Defined in main.asm)
;	REG_EncoderH	;r16-r31;
;	REG_EncoderL	;r16-r31;
;	REG_EncoderState;r16-r31;
;	REG_EncodeAL	;r16-r31;
;	Ram_StatusPos	;ram	 (defined in main.asm)
;
;clockwise         Counter clockwise
;A ___------__________------__
;B ______------____------_____
;  +0 +1 +2 +3 +0  +3 +2 +1 +0
;              +4  -4
;AB
;00 = +0
;10 = +1
;11 = +2
;01 = +3
;
;

.DSEG

.CSEG

;----------------------------------------------------------------------
;purpose
;	to set up system at startup
;in
;out
;uses
;	al (register r16-31)
QUADInInit:

	;PD3,4,5 are alredy set as inputs
	;Ram is cleared in main.asm

	clr	REG_EncoderH		;1,1 
	clr	REG_EncoderL		;1,1 clear interupt counter

	in 	al,MCUCR		;1,1 Note: This has not been optimized for clarity.
	ori	al,(1<<ISC01)		;1,1 Int0 EdgeTriggered  
	andi	al,~(1<<ISC00)		;1,1 Int0 Falling edge
	ori	al,(1<<ISC11)		;1,1 Int1 EdgeTriggered
	ori	al,(1<<ISC10)		;1,1 Int1 rising edge
	out	MCUCR,al		;1,1 

	in	REG_EncoderState,PIND		;1,1 get encoder input. PD 3,4
	andi	REG_EncoderState,0b00011000	;1,1 and off other bits

	in	al,GIMSK			;1,1 
	ori	al,(1<<INT0)|(1<<INT1)		;1,1 Int0 and INT1 IRQ enabled
	out	GIMSK,al			;1,1
	
	ret;

;----------------------------------------------------------------------------
;purpose
;	to clear the current encoder count
;in
;out
;	ram Ram_StatusPos[4] (LSO)
;uses
;	REG_EncoderL,H
;	ax,Bx,Zx (registers r16-31)
QuadClearCounter:

	cli				;1,1  no interupts
	clr	REG_EncoderH		;1,1 
	clr	REG_EncoderL		;1,1 clear interupt count
	sei				;1,1 Enable interrupts

	clr	al
	LDZ	Ram_StatusPos		;2,2  ZH,Zl -> QuadEncoder[4] (macro)
	st	z+,al			;1,2  
	st	z+,al			;1,2  Clear 32 bit count
	st	z+,al			;1,2  
	st	z+,al			;1,2  QuadEncoder[4] = Ax.Yx
	
	ret;

;---------------------------------------------------------------------------------
;purpose
;	to read the current encoder count	
;in
;out
;	ram Ram_StatusPos[4] (LSO)
;uses
;	REG_EncoderL,H
;	Ax,Bx,Yx,Zx (registers r16-31)
QuadReadCounter:

;hack missing IRQ?
	in	al,PIND			;1,1 get encoder input bits 3,4 (Before thay change again)
	mov	ah,REG_EncoderState	;1,1 State = state Xor pins
	eor	al,ah
	andi	al,0b00011000		;1,1 and off other bit
	cpi	al,0
	breq	QuadHackIqmis
	cli				;1,1  no interupts
	rcall	EXT_INT0		;call it like a sub
	sei			; Enable interrupts
QuadHackIqmis:

;-- get count
QuadReadChanged:
	mov	al,REG_EncoderL		;1,1 for compare
	in	ah,PinD			;1,1
	mov	YH,REG_EncoderH		;1,1
	mov	Yl,REG_EncoderL  	;1,1 
	cp	al,Yl
	brne	QuadReadChanged		;1,1-2 if count changed then re-read
	andi	ah,0b00011000		;1,1 Bits 3,4 AB

;-- Add offset (this could be optimized)
	cpi	ah,0b00010000		;1,1  AB 10
	brne	QuadReadOffset10	;1,1-2
	adiw	YL,1			;1,2 Yx = yX +1 (16 bit add)
QuadReadOffset10:

	cpi	ah,0b00011000		;1,1  AB 11?
	brne	QuadReadOffset11	;1,1-2 if eq then
	adiw	YL,2			;1,2 Yx = yX +2 (16 bit add)
QuadReadOffset11:

	cpi	ah,0b00001000		;1,1  AB 01?
	brne	QuadReadOffset01	;1,1-2 if eq then
	adiw	YL,3			;1,2 Yx = yX +3 (16 bit add)
QuadReadOffset01:

;-- expand Yx to 32 bits

	LDZ	Ram_StatusPos		;2,2  ZH,Zl -> Encoder[4] (macro)
	ld	ah,z+			;1,2  H ax.bx = Encoder[4]
	ld	al,z+			;1,2  
	ld	bh,z+			;1,2  
	ld	bl,z+			;1,2  L 

	mov	ch,bh			;last
	eor	ch,yh			;xor with new_H
	sbrs	ch,7	
	rjmp	QuadReadNewDone		; high bit has not changed

	cpi	YH,64			;
	brsh 	QuadReadNewNPos
	subi	al,low(-1)		;1,1  +1
	sbci	ah,high(-1)		;1,1 	if (new >=0) and (old < 0)  then increment high word
QuadReadNewNPos: 			

	cpi	YH,192
	brlo 	QuadReadNewNNeg		;
	subi	al,low(1)		;1,1  -1
	sbci	ah,high(1)		;1,1  if (new <0) and (old >= 0) then decrement high word
QuadReadNewNNeg:

QuadReadNewDone:

;-- save 32 bit count

	LDZ	Ram_StatusPos		;2,2  ZH,Zl -> Encoder[4] (macro)
	st	z+,ah			;1,2 H  QuadEncoder[4] = Ax.Yx 
	st	z+,al			;1,2  
	st	z+,yh			;1,2 
	st	z+,yl			;1,2 L
	ret;

;------------------- quadrature input --------------------------------------
;purpose
;		
;in
;out
;uses
;	REG_Int0sreg		;Sreg
;	REG_EncodeAL		;tempory value
;	REG_EncoderState	;last bit state
;	REG_EncoderH,L		;Encoder 16 bit count
;
;Cycles
;	8 IRQ overhead
;	15 code
;	23 total
;with a 6 cycle latency this can count
; (1/((23+6) * (1/8Mhz) * 2)) * 4 = 551K transitions per second

EXT_INT0: ;Falling edge			;0,4	Encoder B has changed state
EXT_INT1: ;Rising edge

	in	REG_Int0sreg,sreg	;1,1 Save status register
	
	in	REG_EncodeAL,PIND	;1,1 get encoder input bits 3,4 (Before thay change again)
	andi	REG_EncodeAL,0b00011000	;1,1 and off other bits
	
	sbrc	REG_EncodeAL,4		;1,1 Skip if PD4 "encoder A" is cleared(low)
	rjmp	EXT_INTAHigh		;1,1-2 if (A is high) then done
	
	eor	REG_EncoderState,REG_EncodeAL ;1,1 State = state Xor pins
	sbrs	REG_EncoderState,3	;1,1  Skip if (PD3 xor old) is high
	rjmp	EXT_INTDone		;1,1-2 If B has not changed then done

	sbrs	REG_EncodeAL,3		;1,1  Skip if PD3  is set (high)
	rjmp	EXT_INTBLow		;1,1-2 B is low
;EXT_INTBHigh:	

	subi	REG_EncoderL,low(4)	;1,1 Encoder16 = Encoder16 - 4
	sbci	REG_EncoderH,high(4)	;1,1  -4 when "B has changed" and "B is high" and "A is low"
;--- Done
	mov	REG_EncoderState,REG_EncodeAL ;1,1 save State for next time
	out	sreg,REG_Int0sreg	;1,1
	RETI				;1,4 Done

EXT_INTBLow:	

	subi	REG_EncoderL,low(-4)	;1,1 +4 when "B has changed" and "B is low" and "A is low"
	sbci	REG_EncoderH,high(-4)	;1,1 Encoder16 = Encoder16 + 4

EXT_INTDone:
;--- Done
	mov	REG_EncoderState,REG_EncodeAL ;1,1 save State for next time
	out	sreg,REG_Int0sreg	;1,1
	RETI				;1,4

EXT_INTAHigh:
	;
	;error correction code. (if needed)
	;
	;If the last (A low) (B transition) was clockwise
	; and this (A High) (B trasnition) is Counter clockwise
	; then adjust count

	;If the last (A low) (B transition) was counter clockwise
	; and this (A High) (B trasnition) is clockwise
	; then adjust count

;--- Done
	mov	REG_EncoderState,REG_EncodeAL ;1,1 save State for next time
	out	sreg,REG_Int0sreg	;1,1
	RETI				;1,4




